// The following code queries and displays information about
// the blue gene machine.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "llapi.h"

const char* bpStateString(int);
const char* switchStateString(int);
const char* wireStateString(int);
const char* partitionStateString(int);
const char* dimmString(int dim);

main(int argc, char *argv[])
{
	LL_element *queryObject;
	LL_element *machine;
	LL_element *bp;
	LL_element *wire;
	LL_element *swt;
	LL_element *conn;
	char buf1[256];
	char buf2[256];
	const char *partid;
	const char *partstate;
	char *BP_id;
	char *switch_id;
	char *wire_id;
	char *current_partition;
	char *from_comp;
	char *to_comp;
	int obj_count;
	int err_code;
	int rc;
	int *size;
	int count;
	int state;
	int current_partition_state;
	int boolean;
	int from_port;
	int to_port;
	int dim;

	// Initialize the query blue gene
	queryObject = ll_query(BLUE_GENE);
	if (!queryObject) {
		printf("Query Blue Gene: ll_query() returns NULL.\n");
		 exit(1);
	}

	// Set query parameters to query all blue gene machine data
	rc = ll_set_request(queryObject, QUERY_ALL, NULL, ALL_DATA);
	if (rc) {
		printf("Query Blue Gene: ll_set_request() return code = %d\n", rc);
		exit(1);
	}

	// Get the blue gene machine object. Only one machine object 
	// is returned
	err_code = 0;
	obj_count = 0;
	machine = ll_get_objs(queryObject, LL_CM, NULL, &obj_count, &err_code);
	if (machine == NULL || obj_count != 1) {
		printf("Query Blue Gene: ll_get_objs() error code = %d\n", err_code);
		exit(1);
	}

	// Get data from blue gene machine
	rc = ll_get_data(machine, LL_BgMachineSize, &size);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineSize rc = %d\n",rc);
		exit(1);
	}
	printf("Machine size = %dx%dx%d\n",size[0], size[1], size[2]);
	free(size);

	rc = ll_get_data(machine, LL_BgMachineBPSize, &size);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineBPSize rc = %d\n",rc);
		exit(1);
	}
	printf("BP size = %dx%dx%d\n",size[0], size[1], size[2]);
	free(size);

	rc = ll_get_data(machine, LL_BgMachineSwitchCount, &count);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineSwitchCount rc = %d\n",rc);
		exit(1);
	}
	printf("Machine switch count = %d\n",count);

	rc = ll_get_data(machine, LL_BgMachineWireCount, &count);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineWireCount rc = %d\n",rc);
		exit(1);
	}
	printf("Machine wire count = %d\n",count);

	rc = ll_get_data(machine, LL_BgMachinePartitionCount, &count);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachinePartitionCount rc = %d\n",rc);
		exit(1);
	}
	printf("Machine partition count = %d\n",count);
	printf("\n");

	//
	// Base Partitions
	//
	rc = ll_get_data(machine, LL_BgMachineGetFirstBP, &bp);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineGetFirstBP rc = %d\n",rc);
		exit(1);
	}

	while(bp != NULL) {
		rc = ll_get_data(bp, LL_BgBPId, &BP_id);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPId rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(bp, LL_BgBPState, &state);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPState rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(bp, LL_BgBPLocation, &size);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPLocation rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(bp, LL_BgBPSubDividedBusy, &boolean);
		if (rc) {
			printf("Error: ll_get_data LL_BgBPSubDividedBusy rc = %d\n",rc);
			exit(1);
		}

		printf("Base Partition: id=%s state=%-15.15s loc=%dx%dx%d sdb=%s\n",
		      BP_id, bpStateString(state), size[0], size[1], size[2],
		      boolean?"True":"False");
		free(size);
		free(BP_id);

		rc = ll_get_data(machine, LL_BgMachineGetNextBP, &bp);
		if (rc) {
			printf("Error: ll_get_data LL_BgMachineGetNextBP rc = %d\n",rc);
			exit(1);
		}
	}
	printf("\n");

	//
	// Switches
	//
	rc = ll_get_data(machine, LL_BgMachineGetFirstSwitch, &swt);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineGetFirstSwitch rc = %d\n",rc);
		exit(1);
	}

	while(swt != NULL) {
		rc = ll_get_data(swt, LL_BgSwitchId, &switch_id);
		if (rc) {
			printf("Error: ll_get_data LL_BgSwitchId rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(swt, LL_BgSwitchBasePartitionId, &BP_id);
		if (rc) {
			printf("Error: ll_get_data LL_BgSwitchBasePartitionId rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(swt, LL_BgSwitchState, &state);
		if (rc) {
			printf("Error: ll_get_data LL_BgSwitchState rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(swt, LL_BgSwitchDimension, &dim);
		if (rc) {
			printf("Error: ll_get_data LL_BgSwitchDimension rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(swt, LL_BgSwitchConnCount, &count);
		if (rc) {
			printf("Error: ll_get_data LL_BgSwitchConnCount rc = %d\n",rc);
			exit(1);
		}

		printf("Switch: id=%s BP_id=%s state=%-7.7s dim=%s conns=%d\n",
			switch_id, BP_id, switchStateString(state), dimmString(dim), count);
		free(switch_id);
		free(BP_id);

		//
		// Switch Connections
		//
		rc = ll_get_data(swt, LL_BgSwitchGetFirstConn, &conn);
		if (rc) {
			printf("Error: ll_get_data LL_BgSwitchGetFirstConn rc = %d\n",rc);
			exit(1);
		}

		while(conn != NULL) {
			rc = ll_get_data(conn, LL_BgPortConnToSwitchPort, &to_port);
			if (rc) {
				printf("Error: ll_get_data LL_BgPortConnToSwitchPort rc = %d\n",rc);
				exit(1);
			}

			rc = ll_get_data(conn, LL_BgPortConnFromSwitchPort, &from_port);
			if (rc) {
				printf("Error: ll_get_data LL_BgPortConnFromSwitchPort rc = %d\n",rc);
				exit(1);
			}

			rc = ll_get_data(conn, LL_BgPortConnCurrentPartition, &current_partition);
			if (rc) {
				printf("Error: ll_get_data LL_BgPortConnCurrentPartition rc = %d\n",rc);
				exit(1);
			}

			rc = ll_get_data(conn, LL_BgPortConnCurrentPartitionState, &current_partition_state);
			if (rc) {
				printf("Error: ll_get_data LL_BgPortConnCurrentPartitionState rc = %d\n",rc);
				exit(1);
			}

			if (strlen(current_partition) == 0) { 
				partid = "NONE"; 
				partstate = "NONE"; 
			} else {
				partid = current_partition;
				partstate = partitionStateString(current_partition_state);
			}

			printf("\tConnection: from=%d to=%d partition=%-12.12s state=%s\n",
				from_port, to_port, partid, partstate);
			free(current_partition);

			rc = ll_get_data(swt, LL_BgSwitchGetNextConn, &conn);

			if (rc) {
				printf("Error: ll_get_data LL_BgSwitchGetNextConn rc = %d\n",rc);
				exit(1);
			}
		}

		rc = ll_get_data(machine, LL_BgMachineGetNextSwitch, &swt);
		if (rc) {
			printf("Error: ll_get_data LL_BgMachineGetNextSwitch rc = %d\n",rc);
			exit(1);
		}
	}
	printf("\n");

	//
	// Wires
	//
	rc = ll_get_data(machine, LL_BgMachineGetFirstWire, &wire);
	if (rc) {
		printf("Error: ll_get_data LL_BgMachineGetFirstWire rc = %d\n",rc);
		exit(1);
	}

	while(wire != NULL) {
		rc = ll_get_data(wire, LL_BgWireId, &wire_id);
		if (rc) {
			printf("Error: ll_get_data LL_BgWireId rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(wire, LL_BgWireState, &state);
		if (rc) {
			printf("Error: ll_get_data LL_BgWireState rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(wire, LL_BgWireFromPortCompId, &from_comp);
		if (rc) {
			printf("Error: ll_get_data LL_BgWireFromPortCompId rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(wire, LL_BgWireToPortCompId, &to_comp);
		if (rc) {
			printf("Error: ll_get_data LL_BgWireToPortCompId rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(wire, LL_BgWireFromPortId, &from_port);
		if (rc) {
			printf("Error: ll_get_data LL_BgWireFromPortId rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(wire, LL_BgWireToPortId, &to_port);
		if (rc) {
			printf("Error: ll_get_data LL_BgWireToPortId rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(wire, LL_BgWireCurrentPartition, &current_partition);
		if (rc) {
			printf("Error: ll_get_data LL_BgWireCurrentPartition rc = %d\n",rc);
			exit(1);
		}

		rc = ll_get_data(wire, LL_BgWireCurrentPartitionState, &current_partition_state);
		if (rc) {
			printf("Error: ll_get_data LL_BgWireCurrentPartitionState rc = %d\n",rc);
			exit(1);
		}

		if (strlen(current_partition) == 0) { 
			partid = "NONE"; 
			partstate = "NONE"; 
		} else {
			partid = current_partition;
			partstate = partitionStateString(current_partition_state);
		}
		sprintf(buf1,"%s:%d",from_comp,from_port);
		sprintf(buf2,"%s:%d",to_comp,to_port);
		printf("Wire: id=%-16.16s state=%-7.7s from=%-8.8s to=%-8.8s p=%-8.8s pstate=%s\n",
			wire_id, wireStateString(state),buf1, buf2, partid, partstate);
		free(wire_id);
		free(from_comp);
		free(to_comp);
		free(current_partition);

		rc = ll_get_data(machine, LL_BgMachineGetNextWire, &wire);
		if (rc) {
			printf("Error: ll_get_data LL_BgMachineGetNextWire rc = %d\n",rc);
			exit(1);
		}
	}


	// Free objects obtained from Negotiator
	ll_free_objs(queryObject);
	// Free query element
	ll_deallocate(queryObject);
}

const char* bpStateString(int state) {
	char* str_state;
	switch ( state ) {
	case BG_BP_UP:
		str_state = "UP";
		break;
	case BG_BP_DOWN:
		str_state = "DOWN";
		break;
	case BG_BP_MISSING:
		str_state = "MISSING";
		break;
	case BG_BP_ERROR:
		str_state = "ERROR/SOME_DOWN";
		break;
	case BG_BP_NAV:
		str_state = "NAV";
		break;
	default:
		str_state = "<unknown>";
		break;
	}
	return str_state;
}

const char* partitionStateString(int state) {
  char* str_state;
  switch ( state ) {
    case BG_PARTITION_FREE:
      str_state = "FREE";
      break;
    case BG_PARTITION_CONFIGURING:
      str_state = "CFG";  
      break;        
    case BG_PARTITION_READY:
      str_state = "READY";
      break;
    case BG_PARTITION_BUSY:
      str_state = "BUSY";
      break;
    case BG_PARTITION_DEALLOCATING:
      str_state = "DEALC";
      break;
    case BG_PARTITION_ERROR:
      str_state = "ERROR";
      break;
    case BG_PARTITION_NAV:
      str_state = "NAV";
      break;
    default:
      str_state = "<unknown>";
      break;
  }
  return str_state;
}

const char* switchStateString(int state) {
  char* str_state;
  switch ( state ) {
    case BG_SWITCH_UP:
      str_state = "UP";
      break;
    case BG_SWITCH_DOWN:
      str_state = "DOWN";
      break;
    case BG_SWITCH_MISSING:
      str_state = "MISSING";
      break;
    case BG_SWITCH_ERROR:
      str_state = "ERROR";
      break;
    case BG_SWITCH_NAV:
      str_state = "NAV";
      break;
    default:
      str_state = "<unknown>";
      break;
  }
  return str_state;
}

const char* wireStateString(int state) {
  char* str_state;
  switch ( state ) {
    case BG_WIRE_UP:
      str_state = "UP";
      break;
    case BG_WIRE_DOWN:
      str_state = "DOWN";
      break;
    case BG_WIRE_MISSING:
      str_state = "MISSING";
      break;
    case BG_WIRE_ERROR:
      str_state = "ERROR";
      break;
    case BG_WIRE_NAV:
      str_state = "NAV";
      break;
    default:
      str_state = "<unknown>";
      break;
  }
  return str_state;
}

const char* dimmString(int dim) {
  char* str_dim;
  switch ( dim ) {
    case BG_DIM_X:
      str_dim = "X";
      break;
    case BG_DIM_Y:
      str_dim = "Y";
      break;
    case BG_DIM_Z:
      str_dim = "Z";
      break;
    case BG_DIM_NAV:
      str_dim = "NAV";
      break;
    default:
      str_dim = "<unknown>";
      break;
  }
  return str_dim;
}

